package be.iminds.ilabt.jfed.testing.shared;

import be.iminds.ilabt.jfed.testing.shared.AutomatedTestExternalExecutableLocations;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;

/**
 * AutomatedTestExternalExecutableLocationsImpl
 */
public class AutomatedTestExternalExecutableLocationsImpl implements AutomatedTestExternalExecutableLocations {
    private static final Logger LOG = LoggerFactory.getLogger(AutomatedTestExternalExecutableLocationsImpl.class);

    @Nonnull private final Map<String, File> exeMapping;
    @Nullable private final PropConfig config;


    public static interface PropConfig {
        @Nullable String getProperty(@Nonnull String key);
    }

    public AutomatedTestExternalExecutableLocationsImpl(@Nullable PropConfig config) {
        this.exeMapping = new HashMap<>();
        this.config = config;
    }

    @Inject
    public AutomatedTestExternalExecutableLocationsImpl() {
        this.exeMapping = new HashMap<>();
        this.config = null;
    }

    @Nonnull
    @Override
    public File getExe(@Nonnull String exeName) {
        if (!exeMapping.containsKey(exeName))
            throw new IllegalStateException("bug: requireExe(\""+exeName+"\") should have been called earlier");
        return exeMapping.get(exeName);
    }

    @Override
    public void requireExe(@Nonnull String exeName) throws ExecutableNotFoundException {
        File exeLoc = findExecutableLocation(exeName, this.config);
        exeMapping.put(exeName, exeLoc);
    }

    @Nonnull
    private static File findExecutableLocation(@Nonnull String command,
                                               @Nullable PropConfig config)
            throws ExecutableNotFoundException {
        String res;

        //first use what config specifies, if present
        String key = command.toLowerCase().replaceAll("[^a-zA-Z0-9]*", "");
        String exe = config == null ? null : config.getProperty(key+"Exe");
        if (exe != null)
            res = exe;
        else
            res = null;

        //as fallback, look it up using "which"
        if (res == null) {
            res = getWhichOutput(command);
        }

        if (res == null) {
            throw new ExecutableNotFoundException("Executable for \""+command+"\" was not found.");
        }

        File resFile = new File(res);
        if (!resFile.exists()) {
            throw new ExecutableNotFoundException("Executable for \""+command+"\" at \""+res+"\" was not found.");
        }

        return resFile;
    }

    @Nullable
    private static String getWhichOutput(@ Nonnull String command) {
        try {
            //quick and dirty...
            Process cmdProc = Runtime.getRuntime().exec("which "+command);

            String out = "";

            BufferedReader stdoutReader = new BufferedReader(
                    new InputStreamReader(cmdProc.getInputStream()));
            String line;
            while ((line = stdoutReader.readLine()) != null)
                out += line;
            stdoutReader.close();

            BufferedReader stderrReader = new BufferedReader(
                    new InputStreamReader(cmdProc.getErrorStream()));
            while ((line = stderrReader.readLine()) != null)
                out += line;
            stderrReader.close();

            try {
                cmdProc.waitFor();
            } catch (InterruptedException e) {
                LOG.warn("InterruptedException in getWhichOutput", e);
            }

            int retValue = cmdProc.exitValue();
            if (retValue != 0) {
                return null;
            } else {
                return out;
            }
        } catch (Exception e) {
            LOG.error("Failure during \"which "+command+"\". Will assume command is not found.", e);
            return null;
        }
    }
}
